"""
This module contains some useful decorators.
"""
import functools
import time


# Decorator to measure the time spent for a function
def timer(func):
    """Print the runtime of the decorated function"""
    @functools.wraps(func)
    def wrapper_timer(*args, **kwargs):
        start_time = time.perf_counter()
        value = func(*args, **kwargs)
        end_time = time.perf_counter()
        run_time = end_time - start_time
        print(f"Finished {func.__name__!r} in {run_time:.4f} secs")
        return value

    return wrapper_timer


# Decorator to print the function signature and return value
def debug(debug_mode=True):
    """Print the function signature and return value.
       This decorator accepts one argument to set if debug is True/False
    """
    def debug_wrapper(func):
        @functools.wraps(func)
        def wrapper_debug(*args, **kwargs):
            args_repr = [repr(a) for a in args]  # 1
            kwargs_repr = [f"{k}={v!r}" for k, v in kwargs.items()]  # 2
            signature = ", ".join(args_repr + kwargs_repr)  # 3
            if debug_mode:
                print(f"Calling {func.__name__}({signature})")
            value = func(*args, **kwargs)
            if debug_mode:
                print(f"{func.__name__!r} returned {value!r}")  # 4
            return value
        return wrapper_debug

    return debug_wrapper



"""
This module is useful to encrypt and decrypt using RSA algorithm.
"""
import os
from Crypto.PublicKey import RSA
from Crypto.Random import get_random_bytes
from Crypto.Cipher import AES, PKCS1_OAEP


class RSACryptex:
    def __init__(self):
        self.recipient_key: RSA.RsaKey = None
        self.session_key = None
        self.cipher_rsa = None
        self.enc_session_key = None

    def encrypt_data(self, clear_data: bytes, public_key: str) -> bytes:
        """
        This example do the following:
        - read the public key
        - create a random AES key of 128 bits
        - encrypt the AES key with the public key
        - encrypt data with the AES key (as the traditional symmetric algorithm does)
        - then write in a file the following information:
          - encrypted AES key (first 16 bits)
          - nonce utilized from the AES cypher
          - tag of the AES cypher
          - AES cypher bytes

        :param clear_data: list of bytes to encrypt
        :param public_key: RSA key used for encryption
        :return: encrypted bytes
        """
        encbytes_out = list()
        # if the key exist don't read
        if self.recipient_key is None:
            self.recipient_key = RSA.import_key(open(public_key).read())
            self.session_key = get_random_bytes(32)

            # Encrypt the session key with the public RSA key
            self.cipher_rsa = PKCS1_OAEP.new(self.recipient_key)
            self.enc_session_key = self.cipher_rsa.encrypt(self.session_key)

        # Encrypt the data with the AES session key
        cipher_aes = AES.new(self.session_key, AES.MODE_EAX)
        ciphertext, tag = cipher_aes.encrypt_and_digest(clear_data)
        encbytes_out = bytearray()
        for x in (self.enc_session_key, cipher_aes.nonce, tag, ciphertext):
            encbytes_out.extend(x)
        return bytes(encbytes_out)

    def decrypt_data(self, enc_data: bytes, private_key: str, passprhase=None) -> bytes:
        """
        Decrypt data doing the following:
        - get encrypted key and nonce and tag + decrypted bytes
        - decrypt the AES encrypted key using the private key
        - use the AES keys and other information to decrypt data

        :param enc_data: encrypted bytes
        :param private_key: RSA private key used for decryption
        :return: list of decrypted bytes
        """
        # load the RSA private key
        if self.recipient_key is None:
            self.recipient_key = RSA.import_key(open(private_key).read(), passphrase=passprhase)
            # Decrypt the session key with the private RSA key
            self.cipher_rsa = PKCS1_OAEP.new(self.recipient_key)
            # get AES encrypted key
            enc_session_key = enc_data[:self.recipient_key.size_in_bytes()]
            # decrypt AES key
            self.session_key = self.cipher_rsa.decrypt(enc_session_key)

        # get the single elements from bytes list
        nonce = enc_data[self.recipient_key.size_in_bytes():self.recipient_key.size_in_bytes() + 16]
        tag = enc_data[self.recipient_key.size_in_bytes() + 16:self.recipient_key.size_in_bytes() + 32]
        ciphertext = enc_data[self.recipient_key.size_in_bytes() + 32:]

        # Decrypt the data with the AES session key
        cipher_aes = AES.new(self.session_key, AES.MODE_EAX, nonce)
        data = cipher_aes.decrypt_and_verify(ciphertext, tag)
        return data

    @classmethod
    def create_keys(cls, folder: str, passprhase=None):
        """
        Create a public key and private key pair
        :param folder: directory where to create the pycryptex_key and pycryptex_key.pub files
        :return: None
        """
        key = RSA.generate(2048)
        private_key = None
        if passprhase:
            private_key = key.export_key(passphrase=passprhase, pkcs=8,
                                         protection="scryptAndAES128-CBC")
        else:
            private_key = key.export_key()

        with open(os.path.join(folder, "pycryptex_key"), "wb") as file_out:
            file_out.write(private_key)

        public_key = key.publickey().export_key()
        with open(os.path.join(folder, "pycryptex_key.pub"), "wb") as file_out:
            file_out.write(public_key)

    @classmethod
    def is_privatekey_protected(cls, key_file: str) -> bool:
        try:
            private_key = RSA.import_key(open(key_file).read(), passphrase="")
            public_key = private_key.publickey().export_key()
            return False
        except ValueError as e:
            return True
        except Exception as e:
            raise e



from Crypto.Random import get_random_bytes
from Crypto.Protocol.KDF import PBKDF2
from Crypto.Cipher import AES


class AESCryptex:
    def __init__(self):
        self.key: bytes = None
        self.salt: bytes = None

    def encrypt_data(self, clear_data: bytes, pwd: str) -> bytes:
        """
        This example do the following:
        - generate a salt
        - encrypt the clear_data

        :param clear_data: list of bytes to encrypt
        :param pwd: password to encrypt
        :return: encrypted bytes
        """
        if self.salt is None:
            self.salt = get_random_bytes(32)
            self.key = PBKDF2(pwd, self.salt, dkLen=32)  # the key that you can encrypt with
        cipher_aes = AES.new(self.key, AES.MODE_EAX)  # EAX mode
        # Encrypt and digest to get the ciphered data and tag
        ciphered_data, tag = cipher_aes.encrypt_and_digest(clear_data)

        encbytes_out = bytearray()
        for x in (self.salt, cipher_aes.nonce, tag, ciphered_data):
            encbytes_out.extend(x)
        return bytes(encbytes_out)

    def decrypt_data(self, enc_data: bytes, pwd: str) -> bytes:
        """
        This example do the following:
        - retrieve the salt
        - decrypt the clear_data

        :param enc_data: list of bytes to decrypt
        :param pwd: password to encrypt
        :return: decrypted bytes
        """
        if self.salt is None:
            self.salt = enc_data[:32]
            self.key = PBKDF2(pwd, self.salt, dkLen=32)  # the key that you can decrypt with
        # salt = enc_data[:32]
        nonce = enc_data[32:48]
        tag = enc_data[48:64]
        ciphered_data = enc_data[64:]
        # generate the key
        # key = PBKDF2(pwd, salt, dkLen=32)  # the key that you can encrypt with

        cipher = AES.new(self.key, AES.MODE_EAX, nonce)
        original_data = cipher.decrypt_and_verify(ciphered_data, tag)
        return original_data




"""
Utils module for repetitive jobs.
"""
import os
import sys
from pathlib import Path
import subprocess
import pycryptex
from os import path
import toml
import click


def get_home() -> str:
    return str(Path.home())


def create_home_folder():
    """
    If it does not exist $HOME/.pycryptex folder will be created
    :return:
    """
    pycryptex_folder = os.path.join(get_home(), '.pycryptex')
    if not os.path.exists(pycryptex_folder):
        os.mkdir(pycryptex_folder)
        return True, pycryptex_folder
    return False, pycryptex_folder


def create_config() -> bool:
    """
    If it does not exist $HOME/.pycryptex/pycryptex.toml file will be created
    :return:
    """
    # first check to create $HOME/.pycryptex folder
    create_home_folder()
    pycryptex_config_file = os.path.join(get_home(), '.pycryptex', 'pycryptex.toml')

    if not os.path.exists(pycryptex_config_file):
        with open(pycryptex_config_file, "w") as f:
            f.write("""# Configuration file for pycryptex
[config]
# path to the pager application where to see decrypted file
pager = "vim"
# default private key for RSA decryption
private-key = ""
# default public key for RSA encryption
public-key = ""
""")
            return True
    return False


def read_config():
    config_path = os.path.join(get_home(), '.pycryptex', 'pycryptex.toml')
    if path.exists(config_path):
        pycryptex.config_file = toml.load(config_path)
    else:
        pycryptex.config_file = {
            "config": {
                'pager': 'vim',
                'private-key': "",
                'public-key': "",
            }
        }


def open_pager(config, f: str):
    # load config file first
    read_config()
    if config.verbose:
        click.echo(click.style(f"config_file loaded: {pycryptex.config_file}", fg="magenta", bold=True))
    exit_code = subprocess.call([pycryptex.config_file['config']['pager'], f])
    if exit_code != 0:
        click.echo(click.style(f"Houston, we have a problem: the opened subprocess has a return value equal to"
                               f" {exit_code}", fg="red", bold=True))


def count_file(path, no_nested: bool) -> int:
    """
    Count the file in a folder and its nested folders
    :param path: directory where begins to count
    :return: total files number
    """
    i = 0
    if no_nested:
        currentDirectory = Path(path)
        for currentFile in currentDirectory.iterdir():
            if currentFile.is_file():
                i += 1
        return i
    else:
        for root, d_names, f_names in os.walk(path):
            for f in f_names:
                i += 1
        return i


def is_valid_path(path) -> bool:
    # test first for file existence
    if not os.path.exists(path):
        click.echo(click.style(f"👍 Nothing to do, file or folder {path} doesn't exist!", fg="yellow", bold=False))
        return False
    return True



import os
from pathlib import Path
import sys
from getpass import getpass
from pycryptex.crypto.rsa import RSACryptex
import pycryptex
from os import path
import click
from tqdm import tqdm
from pycryptex.internal import utils
from pycryptex.internal.decorators import timer, debug
from pycryptex.crypto import common
from pycryptex.crypto.aes import AESCryptex


class Config():
    """
    This class is to pass some configuration trough the commands
    """

    def __init__(self):
        pass


# Decorator that will create a new instance of Config class. The instance is config and the object can be passed
# through the commands
pass_config = click.make_pass_decorator(Config, ensure=True)


@click.group()
@click.version_option(version=None, message="pycryptex CLI application (version: %(version)s)")
@click.option('--verbose', "-v", is_flag=True, help='bool, to specify if needed a verbose mode')
@pass_config
def cli(config, verbose):
    """
    This command is executed before other commands in the same group. The group commands
    are between the main command (pycrypto) and the single commands.

    For instance:
    pycryptex --verbose encrypt test/appway.png

    Using config.verbose is possible to pass verbose from a command to another
    """
    config.verbose = verbose


@cli.command()
@click.argument('file', required=True)
@click.option('--pubkey', default="", help='(optional) specify the RSA public key')
@click.option('--keep', '-k', is_flag=True, default=False,
              help="(optional, bool=False) if specified, do not remove the original file")
@click.option('--no-nested', is_flag=True, default=False,
              help="(optional, bool=False) in case FILE is a folder, specify it to avoid encrypting the nested folders")
@pass_config
def encrypt(config, file, pubkey, keep, no_nested):
    """Encrypt files or folders using RSA/AES algorithms"""
    # test first for file/folder existence
    if not utils.is_valid_path(file):
        return
    try:
        # in case of pubkey is not passed, pycryptex calculates the default path
        pubkey = load_key(pubkey, 'public-key', 'pycryptex_key.pub')
        rsa: RSACryptex = RSACryptex()
        # check if the file param is a file or a dir
        if os.path.isdir(file):
            encrypt_decrypt_folder(rsa.encrypt_data, True, folder=file, keep=keep, no_nested=no_nested,
                                   public_key=pubkey)
            click.echo(click.style(f"👍 Folder encrypted successfully! [key used: {pubkey}]", fg="green", bold=True))
        else:
            # encryption of the file
            f, done = common.encrypt_file(file=file, func=rsa.encrypt_data, remove=not keep, public_key=pubkey)
            if done:
                click.echo(click.style(f"👍 File encrypted successfully in {f}! [key used: {pubkey}]", fg="green", bold=True))
            else:
                click.echo(click.style(f"👍 Nothing to do, file already encrypted!", fg="yellow", bold=False))

        if config.verbose:
            click.echo(click.style(f"pubkey used is: {pubkey}", fg="magenta", bold=False))
            click.echo(click.style(f"config_file loaded: {pycryptex.config_file}", fg="magenta", bold=True))
    except Exception as e:
        click.echo(click.style(f"● Houston, help: {e}, {type(e)}", fg="red", bold=True))
        sys.exit(2)


@cli.command()
@click.argument('file', required=True)
@click.option('--privkey', default="", help='(optional) specify the RSA private key')
@click.option('--keep', '-k', is_flag=True, default=False,
              help="(optional, bool=False) if specified, do not remove the original file")
@click.option('--pager', '-p', is_flag=True,
              help="(optional, bool=False) open the pager to read decrypted file (only if the FILE arg is a file)")
@click.option('--no-nested', is_flag=True, default=False,
              help="(optional, bool=False) in case FILE is a folder, specify it to avoid decrypting the nested folders")
@pass_config
def decrypt(config, file, privkey, keep, pager, no_nested):
    """Decrypt files or folders using RSA/AES algorithms"""
    # test first for file/folder existence
    if not utils.is_valid_path(file):
        return
    try:
        f = ""
        privkey = load_key(privkey, 'private-key', 'pycryptex_key')
        # check if the private key has a password
        passphrase = None
        if RSACryptex.is_privatekey_protected(privkey):
            passphrase = getpass("Please insert your passphrase: ")
        rsa: RSACryptex = RSACryptex()
        if os.path.isdir(file):
            encrypt_decrypt_folder(rsa.decrypt_data, False, folder=file, keep=keep,
                                   no_nested=no_nested, passprhase=passphrase, private_key=privkey)
            click.echo(click.style(f"👍 Folder decrypted successfully! [key used: {privkey}]", fg="green", bold=True))
        else:  # single file case
            f, done = common.decrypt_file(file=file, func=rsa.decrypt_data, remove=not keep, passprhase=passphrase,
                                          private_key=privkey)
            if done:
                click.echo(click.style(f"👍 File decrypted successfully in {f}! [key used: {privkey}]", fg="green", bold=True))
            else:
                click.echo(click.style(f"👍 Nothing to do, file already decrypted!", fg="yellow", bold=False))
            # open file in a pager
            if pager:
                utils.open_pager(config, f)

    except ValueError as e:
        click.echo(click.style(f"Houston, help: it is possible that you use the wrong key file to decrypt "
                               f"the document or that the passphrase is incorrect. \nTry with the private key "
                               f"corresponding to the public key used to encrypt the file: {e}", fg="red", bold=True))
        sys.exit(2)
    except Exception as e:
        click.echo(click.style(f"● Houston, help: {e}, {type(e)}", fg="red", bold=True))
        sys.exit(2)


@cli.command()
@pass_config
def create_keys(config):
    """
    Create RSA public/private keys pair into the
    '$HOME/.pycryptex' folder.
    """
    try:
        # does keys exist in the target folder?
        is_created, pycryptex_folder = utils.create_home_folder()
        if is_created:
            click.echo(click.style(f"👍 .pycryptex folder created in: {pycryptex_folder}", fg="green", bold=False))
        if os.path.exists(os.path.join(pycryptex_folder, 'pycryptex_key')) or \
                os.path.exists(os.path.join(pycryptex_folder, 'pycryptex_key.pub')):
            click.echo(click.style(
                "[PAY ATTENTION]\n"
                "The standard keys are present into the default .pycryptex folder. If you confirm to proceed and\n"
                "you already have some document encrypted, you will not be able to open them (if you haven't also copied\n"
                "keys in another location!)", fg="red", bold=True))

        if click.confirm(f"Do you confirm keys creation into {pycryptex_folder}?"):
            passphrase = None
            if click.confirm(f"To make your password more secure, do you like to add a passphrase?"):
                passphrase = getpass("Please insert your passphrase: ")
                passprhase2 = getpass("Please confirm your passphrase: ")
                if passphrase != passprhase2:
                    raise Exception('passwords doesn\'t match!')
            # creation of the keys
            RSACryptex.create_keys(pycryptex_folder, passphrase)
            click.echo(
                click.style("New keys created successfully! Now you can use the other commands, happy encryption!",
                            fg="green", bold=True))
        else:
            click.echo("Keys creation aborted by the user")
    except Exception as e:
        click.echo(
            click.style(f"● Houston, we have a problem during the creation of the keys: {e}", fg="red", bold=True))
        sys.exit(2)


@cli.command()
@pass_config
def create_config(config):
    """
    Create the config file in the $HOME/.pycryptex folder if the file doesn't exist
    """
    try:
        if utils.create_config():
            click.echo(click.style(f"👍 pycryptex.toml file created in: "
                                   f"{os.path.join(utils.get_home(), '.pycryptex', 'pycryptex.toml')}", fg="green",
                                   bold=False))
        else:
            click.echo(click.style(f"👍 Nothing to do, file "
                                   f"{os.path.join(utils.get_home(), '.pycryptex', 'pycryptex.toml')} already exists!",
                                   fg="magenta", bold=False))
    except Exception as e:
        click.echo(click.style(f"● Houston, help: {e}", fg="red", bold=True))
        sys.exit(2)


@cli.command()
@click.argument('file', required=True)
@click.option('--keep', '-k', is_flag=True, default=False,
              help="(optional, bool=False) if specified, do not remove the original file")
@click.option('--no-nested', is_flag=True, default=False,
              help="(optional, bool=False) in case FILE is a folder, specify it to avoid encryption of "
                   "the nested folders")
@pass_config
def encrypt_aes(config, file, keep, no_nested):
    """Encrypt files or folders using AES encryption"""
    try:
        encrypt_decrypt_aes(config, file, keep, no_nested, True)
    except Exception as e:
        click.echo(click.style(f"● Houston, help: {e}, {type(e)}", fg="red", bold=True))
        sys.exit(2)


@cli.command()
@click.argument('file', required=True)
@click.option('--keep', '-k', is_flag=True, default=False,
              help="(optional, bool=False) if specified, do not remove the original file")
@click.option('--no-nested', is_flag=True, default=False,
              help="(optional, bool=False) in case FILE is a folder, specify it to avoid decryption of "
                   "the nested folders")
@pass_config
def decrypt_aes(config, file, keep, no_nested):
    """Decrypt files or folders using AES encryption"""
    try:
        encrypt_decrypt_aes(config, file, keep, no_nested, False)
    except ValueError as e:
        click.echo(click.style(f"Houston, help: it is possible that the password you used is incorrect! [{e}]",
                               fg="red", bold=True))
        sys.exit(2)
    except Exception as e:
        click.echo(click.style(f"● Houston, help: {e}, {type(e)}", fg="red", bold=True))
        sys.exit(2)


def load_key(key_path: str, key_config_name: str, key_default: str) -> str:
    """
    PyCryptex try to load the RSA private or public keys
    :param key_path: path of the key specified as parameter
    :param key_config_name:
    :param key_default:
    :return:
    """
    if len(key_path) == 0:
        # read config to check if there is a pubkey, else try to load pycryptex_key.pub
        utils.read_config()
        try:
            if len(pycryptex.config_file['config'][key_config_name]) > 0:
                key_path = pycryptex.config_file['config'][key_config_name]
            else:
                key_path = os.path.join(utils.get_home(), '.pycryptex', key_default)
        except KeyError:
            key_path = os.path.join(utils.get_home(), '.pycryptex', key_default)
    # check if the key exists
    if not path.exists(key_path):
        click.echo(
            click.style(f"Houston, help: the key is missing in '{key_path}'", fg="red", bold=False))
        click.echo(f"If you have your own key, pass it as argument or, if you need pycryptex create "
                   "the keys for you, type:\n"
                   "pycryptex create-keys")
        sys.exit(2)
    return  key_path

@timer
def encrypt_decrypt_folder(func, is_encrypt: bool, folder: str, keep: bool, no_nested: bool = False, **kwargs):
    """
    Function to encrypt or decrypt a folder.
    :param is_encrypt:
    :param folder: folder path
    :param key:
    :param keep:
    :param passprhase:
    :return:
    """
    click.echo(click.style(f"● Collecting folder files...", fg="magenta", bold=True))
    total = utils.count_file(folder, no_nested)
    click.echo(click.style(f"Number of files read in {folder} are: {total}", fg="white", bold=True))
    with tqdm(total=total, desc='encryption state' if is_encrypt else 'decryption state') as pbar:
        # in case of no_nested uses the simple read of the first level directory, otherwise walks into all the
        # nested levels
        if no_nested:
            currentDirectory = Path(folder)
            for currentFile in currentDirectory.iterdir():
                if currentFile.is_file():
                    if is_encrypt:
                        common.encrypt_file(str(currentFile), func, remove=not keep, **kwargs)
                    else:
                        common.decrypt_file(str(currentFile), func, remove=not keep, **kwargs)
                    pbar.update(1)
        else:
            for root, dir_names, file_names in os.walk(folder):
                for f in file_names:
                    if is_encrypt:
                        common.encrypt_file(os.path.join(root, f), func, remove=not keep, **kwargs)
                    else:
                        common.decrypt_file(os.path.join(root, f), func, remove=not keep, **kwargs)
                    pbar.update(1)


def encrypt_decrypt_aes(config, file, keep, no_nested, is_encryption: bool):
    """Encrypt or decrypt a file using AES encryption"""
    # test first for file/folder existence
    if not utils.is_valid_path(file):
        return
    passphrase = getpass("Please insert your passphrase: ")
    aes = AESCryptex()
    # set var for encryption or decryption
    crypto_func = aes.decrypt_data
    crypto_term = "decrypted"
    if is_encryption:
        crypto_term = "encrypted"
        crypto_func = aes.encrypt_data

    # check if the file param is a file or a dir
    if os.path.isdir(file):
        encrypt_decrypt_folder(crypto_func, is_encryption, folder=file, keep=keep, no_nested=no_nested,
                               pwd=passphrase)
        click.echo(click.style(f"👍 Folder {crypto_term} successfully!", fg="green", bold=True))
    else:
        # encryption/decryption of the file
        if is_encryption:
            f, done = common.encrypt_file(file, crypto_func, remove=not keep, pwd=passphrase)
        else:
            f, done = common.decrypt_file(file, crypto_func, remove=not keep, pwd=passphrase)
        if done:
            click.echo(click.style(f"👍 File {crypto_term} successfully in {f}", fg="green", bold=True))
        else:
            click.echo(click.style(f"👍 Nothing to do, file already {crypto_term}!", fg="yellow", bold=False))

    if config.verbose:
        click.echo(click.style(f"config_file loaded: {pycryptex.config_file}", fg="magenta", bold=True))


if __name__ == '__main__':
    print("main invoked!")
    cli(sys.argv[1:])



import setuptools
from setuptools import setup
import os
from pathlib import Path

"""
This instruction inform setup tool to read our doc file and to include as a long description of the package.
"""
with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="pycryptex",
    author="mas2020",
    author_email="andrea.genovesi@gmail.com",
    version="0.4.0",
    url="https://github.com/mas2020-python/pycryptex",
    description="Python CLI application to easily encrypt and decrypt file and folders. Easy and fast for the lovers"
                " of the CLI.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    packages=setuptools.find_packages(),
    install_requires=[
        'click==7.1.2',
        'pycryptodome==3.9.8',
        'toml==0.10.1',
        'tqdm==4.50.2',
    ],
    entry_points='''
        [console_scripts]
        pycryptex=pycryptex.main:cli
    ''',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6, <4',
)



"""
This module contains some common function used by the crypto modules.
"""
import os


def encrypt_file(file: str, func, remove=False, **kwargs) -> (str, bool):
    """
    Encrypt the file and create a new file appending .enc.

    :param file: file to encrypt
    :param public_key: RSA key used for encryption
    :param remove: bool to specify if remove original file
    :return: None
    """
    # if the file name ends with .enc, return ""
    if file.endswith(".pycpx"):
        return file, False
    with open(file, 'rb') as byte_reader:
        # Read all bytes
        clear_bytes = byte_reader.read(-1)
    # call the func to encrypt and convert the byte arrays into bytes to write on disk
    enc_bytes_list = func(clear_bytes, **kwargs)
    enc_filename = "".join((file, ".pycpx"))
    with open(enc_filename, "wb") as f:
        f.write(enc_bytes_list)
    if remove:
        os.remove(file)
    return enc_filename, True


def decrypt_file(file: str, func, remove=False, **kwargs) -> (str, bool):
    """
    Decrypt the file passed as argument and create a new file removing the .enc extension.

    :param file: encrypted file to decrypt
    :param private_key: RSA private key used for decryption
    :param remove: bool to specify if remove the encrypted file
    :return: the name of the file that has been decrypted
    """
    # if the file name ends with .enc, return ""
    if not file.endswith(".pycpx"):
        return file, False
    with open(file, 'rb') as byte_reader:
        # Read all bytes
        enc_bytes = byte_reader.read(-1)
    clear_bytes_list = func(enc_bytes, **kwargs)
    with open(file[:-6], "wb") as f:
        f.write(clear_bytes_list)
    if remove:
        os.remove(file)
    return file[:-6], True
